# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# Ported by NoMaY from GNURX version to CC-RX version

set(CMAKE_MODULE_PATH "${CMAKE_CURRENT_LIST_DIR}/../../cmake/Modules")

set(CMAKE_SYSTEM_NAME Generic)
set(CMAKE_SYSTEM_PROCESSOR rx) # It seems that this variable is not used actually by CMake.

# default to Release build
if(NOT CMAKE_BUILD_TYPE)
    set(CMAKE_BUILD_TYPE "Debug" CACHE STRING "Choose the type of build, options are: Debug Release." FORCE)
endif()

set(TOOLCHAIN_EXT ".exe")

find_program(COMPILER_ON_PATH "ccrx${TOOLCHAIN_EXT}")

if(COMPILER_ON_PATH) 
    # check on the current path
    get_filename_component(RX_TOOLCHAIN_PATH ${COMPILER_ON_PATH} DIRECTORY)
    string(REPLACE "/" "\\" _RX_TOOLCHAIN_PATH "${RX_TOOLCHAIN_PATH}")
    message(STATUS "Using RX CC-RX from path = ${_RX_TOOLCHAIN_PATH}")
    # check the BIN_RX environment variable for libgen
    if(NOT DEFINED ENV{BIN_RX})
        message(NOTICE
            "Error: The BIN_RX environment variable is not defined.\n"
            "RX CC-RX compiler needs its `bin` folder path in the BIN_RX."
        )
        message(FATAL_ERROR "Aborted due to runtime environment error.")
    elseif(NOT ($ENV{BIN_RX} STREQUAL ${_RX_TOOLCHAIN_PATH}))
        message(NOTICE
            "Error: Other RX CC-RX is specified in the BIN_RX = $ENV{BIN_RX}\n"
            "The same RX CC-RX should be specified in the BIN_RX"
        )
        message(FATAL_ERROR "Aborted due to runtime environment error.")
    endif()
    unset(_RX_TOOLCHAIN_PATH)
else()
    # not found
    message(NOTICE
        "Error: Unable to find RX CC-RX compiler on the PATH = $ENV{Path}\n"
        "RX CC-RX compiler needs its `bin` folder path in the PATH."
    )
    # check the BIN_RX environment variable for libgen
    if(NOT DEFINED ENV{BIN_RX})
        message(NOTICE
            "Error: The BIN_RX environment variable is not defined.\n"
            "RX CC-RX compiler needs its `bin` folder path in the BIN_RX."
        )
    endif()
    message(FATAL_ERROR "Aborted due to runtime environment error.")
endif()

find_program(CONVERTER_ON_PATH "renesas_cc_converter${TOOLCHAIN_EXT}")

if(CONVERTER_ON_PATH) 
    # check on the current path
    get_filename_component(RX_EXTERNAL_TOOLCHAIN_PATH ${CONVERTER_ON_PATH} DIRECTORY)
    string(REPLACE "/" "\\" _RX_EXTERNAL_TOOLCHAIN_PATH "${RX_EXTERNAL_TOOLCHAIN_PATH}")
    message(STATUS "Using e2 studio's X Converter from path = ${_RX_EXTERNAL_TOOLCHAIN_PATH}")
    unset(_RX_EXTERNAL_TOOLCHAIN_PATH)
else()
    # not found
    message(NOTICE
        "Warning: Unable to find e2 studio's X Converter on the PATH = $ENV{Path}\n"
        "Debugging with RX GDB is not available when the converter is not executed."
    )
endif()

# perform compiler test with the static library
#set(CMAKE_TRY_COMPILE_TARGET_TYPE STATIC_LIBRARY)

set(CMAKE_C_COMPILER    ${RX_TOOLCHAIN_PATH}/ccrx${TOOLCHAIN_EXT} CACHE STRING "")
set(CMAKE_CXX_COMPILER  ${RX_TOOLCHAIN_PATH}/ccrx${TOOLCHAIN_EXT} CACHE STRING "")
set(CMAKE_ASM_COMPILER  ${RX_TOOLCHAIN_PATH}/asrx${TOOLCHAIN_EXT})
set(CMAKE_LINKER        ${RX_TOOLCHAIN_PATH}/rlink${TOOLCHAIN_EXT})
set(CMAKE_AR            ${RX_TOOLCHAIN_PATH}/rlink${TOOLCHAIN_EXT})
set(RENESAS_LIBRARY_GENERATOR ${RX_TOOLCHAIN_PATH}/lbgrx${TOOLCHAIN_EXT})
if(RX_EXTERNAL_TOOLCHAIN_PATH)
    set(RENESAS_XCONVERTER  ${RX_EXTERNAL_TOOLCHAIN_PATH}/renesas_cc_converter${TOOLCHAIN_EXT})
endif()

set(CMAKE_FIND_ROOT_PATH_MODE_PROGRAM NEVER)
set(CMAKE_FIND_ROOT_PATH_MODE_LIBRARY ONLY)
set(CMAKE_FIND_ROOT_PATH_MODE_INCLUDE ONLY)
set(CMAKE_FIND_ROOT_PATH_MODE_PACKAGE ONLY)

set(COMMON_FLAGS    "-goptimize -listfile -utf8 -outcode=utf8")
set(CMAKE_C_FLAGS 	"${CPU_FLAGS} ${COMMON_FLAGS} -type_size_access_to_volatile") # FIXME: ${LIBC_INCLUDE} ?
set(CMAKE_CXX_FLAGS "${CPU_FLAGS} ${COMMON_FLAGS} -type_size_access_to_volatile")
set(CMAKE_ASM_FLAGS "${CPU_FLAGS} -goptimize -listfile -utf8")
set(CMAKE_C_LINK_FLAGS     "-noprelink")
set(CMAKE_CXX_LINK_FLAGS   "")
set(CMAKE_EXE_LINKER_FLAGS "-optimize=symbol_delete,branch") # FIXME: ${LD_FLAGS} ?
# TODO: Can -lang=c99 be automatically passed to LIBGEN(i.e.  <FLAGS> in ${CMAKE_${lang}_LINK_EXECUTABLE})?
set(RENESAS_LIBRARY_GENERATOR_C_FLAGS   "${CMAKE_C_FLAGS} -lang=c99 -head=runtime,ctype,math,mathf,stdarg,stdio,stdlib,string,c99_complex,fenv,inttypes,wchar,wctype")
set(RENESAS_LIBRARY_GENERATOR_CXX_FLAGS "${CMAKE_CXX_FLAGS} -lang=c99 -head=all")
set(RENESAS_XCONVERTER_FLAGS "")

set(CMAKE_C_COMPILER_ID RENESAS)
set(CMAKE_C_COMPILER_ID_RUN TRUE)
set(CMAKE_C_COMPILER_ARCHITECTURE_ID RX)
set(CMAKE_C_COMPILER_FORCED TRUE)
set(CMAKE_CXX_COMPILER_ID RENESAS)
set(CMAKE_CXX_COMPILER_ID_RUN TRUE)
set(CMAKE_CXX_COMPILER_ARCHITECTURE_ID RX)
set(CMAKE_CXX_COMPILER_FORCED TRUE)
set(CMAKE_ASM_COMPILER_ID RENESAS)
set(CMAKE_ASM_COMPILER_ID_RUN TRUE)
set(CMAKE_ASM_COMPILER_ARCHITECTURE_ID RX)
set(CMAKE_ASM_COMPILER_FORCED TRUE)
